'user strict';

var pkgInfo   = require('./package.json');
var path      = require('path');
var caller    = require('caller');
var express   = require('express');
var enrouten  = require('express-enrouten');
var bootstrap = require('./lib/bootstrap');
var log       = require('./lib/logger');

/**
 * Create an instance of a microservice and mount upon the parent
 * Express application.
 *
 * @public
 */
module.exports = function (options) {
  var app = express();

  options = buildOptions(options);

  app.once('mount', function onmount(parent) {
    bootstrap(parent, options);

    if (options.debug) {
      log.info(options, 'bootstrapped microservice');
    }
  });

  return app;
};

/**
 * Construct an options instance.
 */
var buildOptions = function(options) {
  options    = options || {};


  // Default options
  options.server                = options.server || { port: process.env.PORT };
  options.debug                 = options.debug || false;
  options.discoverable          = options.discoverable || false;
  options.monitorsPath          = options.monitorsPath || 'monitors';
  options.controllersPath       = options.controllersPath || 'controllers';
  options.callerPath            = options.callerPath || path.dirname(caller(2));
  options.serviceName           = pkgInfo.name;
  options.serviceBasePath       = 'services';
  options.partialResponseQuery  = options.partialResponseQuery || 'fields';
  options.correlationHeaderName = options.correlationHeaderName || 'X-CorrelationID';
  options.validatorOptions      = options.validatorOptions || null;

  // Feature Flags
  options.enableBodyParsing     = options.enableBodyParsing || true;
  options.enableEtag            = options.enableEtag || false;
  options.enableRequestTracing  = options.enableRequestTracing || false;

  // Return now if we have no config
    return options;

};
